from .utils import is_valid_image, is_blurry, load_upsampler, load_face_enhancer
import os
import cv2
from basicsr.utils import imwrite

def enhance_image(input_path, output_dir='results', suffix='restored', upscale=2, use_gfpgan=True):
    os.makedirs(output_dir, exist_ok=True)

    if not is_valid_image(input_path):
        print("Invalid image. Please upload a valid image.")
        return

    if is_blurry(input_path):
        print("Image is too blurry. Please upload another one.")
        return

    print("Loading model...")
    upsampler = load_upsampler(upscale)
    face_enhancer = load_face_enhancer(upsampler, upscale) if use_gfpgan else None

    img_name = os.path.basename(input_path)
    img = cv2.imread(input_path, cv2.IMREAD_COLOR)

    try:
        if use_gfpgan and face_enhancer:
            _, _, output = face_enhancer.enhance(
                img, has_aligned=False, only_center_face=False, paste_back=True
            )
        else:
            output, _ = upsampler.enhance(img, outscale=upscale)
    except Exception as e:
        print(f"Enhancement failed: {e}")
        output = img

    save_name = f"{os.path.splitext(img_name)[0]}_{suffix}.png"
    save_path = os.path.join(output_dir, save_name)
    imwrite(output, save_path)
    print(f"Enhanced image saved to {save_path}")



from .enhancer import enhance_image
from .utils import is_valid_image, is_blurry, load_upsampler, load_face_enhancer

__all__ = [
    "enhance_image",
    "is_valid_image",
    "is_blurry",
    "load_upsampler",
    "load_face_enhancer",
]



MODEL_URLS = {
    "realesrgan": "https://github.com/xinntao/Real-ESRGAN/releases/download/v0.2.1/RealESRGAN_x2plus.pth",
    "gfpgan": "https://github.com/TencentARC/GFPGAN/releases/download/v1.3.0/GFPGANv1.3.pth"
}


import os
import urllib.request
import torch
import numpy as np
from PIL import Image
import cv2
from basicsr.archs.rrdbnet_arch import RRDBNet
from realesrgan import RealESRGANer
from gfpgan import GFPGANer
from model_link import MODEL_URLS

def is_valid_image(image_path):
    if not os.path.isfile(image_path):
        return False
    try:
        with Image.open(image_path) as img:
            img.verify()
        return True
    except:
        return False

def is_blurry(image_path, threshold=4.0):
    img = Image.open(image_path).convert("L")
    img_np = np.array(img)
    laplacian_var = cv2.Laplacian(img_np, cv2.CV_64F).var()
    print(f"Laplacian variance: {laplacian_var:.2f}")
    return laplacian_var < threshold

def download_if_missing(url, dest_path):
    if not os.path.exists(dest_path):
        os.makedirs(os.path.dirname(dest_path), exist_ok=True)
        urllib.request.urlretrieve(url, dest_path)
        print(f"Model Save to: {dest_path}")

def load_upsampler(upscale):
    model_path = "weights/RealESRGAN_x2plus.pth"
    download_if_missing(MODEL_URLS["realesrgan"], model_path)
    model = RRDBNet(
        num_in_ch=3, num_out_ch=3, num_feat=64,
        num_block=23, num_grow_ch=32, scale=2
    )
    use_half = torch.cuda.is_available()
    return RealESRGANer(
        scale=upscale, model_path=model_path, model=model,
        tile=400, tile_pad=10, pre_pad=0, half=use_half
    )

def load_face_enhancer(upsampler, upscale):
    gfpgan_path = 'weights/GFPGANv1.3.pth'
    download_if_missing(MODEL_URLS["gfpgan"], gfpgan_path)
    return GFPGANer(
        model_path=gfpgan_path,
        upscale=upscale,
        arch='clean',
        channel_multiplier=2,
        bg_upsampler=upsampler
    )



from setuptools import setup, find_packages

setup(
    name="image-enhancer-vc",
    version="0.1.1",
    author="Innamuri Ahalya",
    author_email="innamuriahalya1234@gmail.com",
    description="A deep learning-powered image enhancement library using Real-ESRGAN and GFPGAN",
    long_description=open("README.md", encoding="utf-8").read(),
    long_description_content_type="text/markdown",
    packages=find_packages(),
    install_requires=[
        "torch>=1.10.0",
        "opencv-python>=4.5.0",
        "numpy>=1.19.0,<2.0",  
        "Pillow>=9.0.0",
        "basicsr>=1.4.2",
        "realesrgan>=0.3.0",
        "gfpgan>=1.3.8",
        "urllib3<2.0",
    ],
    python_requires=">=3.7",
    include_package_data=True,
)
